(** Time-stamp: <modified the 03/06/2008 (at 14:40) by Erwan Jahier> *)



(**********************************************************************************)

open Lxm
open SyntaxTree
open SyntaxTreeCore

let (build_node_var : var_info srcflagged list -> var_info srcflagged list -> 
      var_info srcflagged list option -> node_vars) =
  fun invars outvars locvars_opt -> 
    let get_var_name vif = vif.it.var_name in
      {
	inlist  = List.map get_var_name invars;
	outlist = List.map get_var_name outvars;
	loclist = (
	  match locvars_opt with
	    | None -> None
	    | Some locvars -> Some (List.map get_var_name locvars)
	);
	vartable = 
	  let tbl = Hashtbl.create 0 in
	  let add_var_in_tbl vif = Hashtbl.add tbl vif.it.var_name vif in
	    List.iter add_var_in_tbl invars;
	    List.iter add_var_in_tbl outvars;
	    (match locvars_opt with
	       | None -> ()
	       | Some locvars -> List.iter add_var_in_tbl locvars
	    );
	    tbl;
      }

(* Une collection de "meta fonctions" pour faciliter la vie *)


(*------------------------------------------------------
flat_flagged_list
--------------------------------------------------------
Entre :
--------
- inlist : ('a list * 'b) list
- makeitem : 'a -> 'b -> c' 
--------------------------------------------------------
Sortie :
--------
- outlist : c' list
--------------------------------------------------------
Effets de bords :
---------------
- aucun en interne
- makeitem est appele de gauche  droite
--------------------------------------------------------
Exemple :
-----------------
flat_flagged_list [ ([a1;a2;a3], b1) ; ([a4;a5], b2) ] f
<=>
let c1 = (f a1 b1) in
let c2 = (f a2 b1) in
let c3 = (f a3 b1) in
let c4 = (f a4 b2) in
let c5 = (f a5 b2) in
[ c1; c2; c3; c4; c5 ]
------------------------------------------------------*)
let flat_flagged_list 
    (inlist:   ('a list * 'b) list)
    (makeitem: 'a -> 'b -> 'c) 
    = (
      (*g: concatene les 'c list*)
      let g (cl: 'c  list) ((al: 'a list) , (b: 'b)) = (
	(*f: fabrique un 'c *)
	let f (a: 'a) = makeitem a b in 
	  List.append cl (List.map f al) 
      ) in 
	(*on folde g sur inlist*)
	List.fold_left g [] inlist
    )

let _ = assert (
  (flat_flagged_list 
     [ (["a1";"a2";"a3"], "b1") ; (["a4";"a5"], "b2") ] 
     (fun a b -> a ^ "-" ^ b))
  = 
    ["a1-b1"; "a2-b1"; "a3-b1"; "a4-b2"; "a5-b2"]
  )

(*------------------------------------------------------
flat_twiced_flagged_list
--------------------------------------------------------
mme principe mais avec deux niveaux de flags :

let mk a b c = (a,b,c)

let toto =
[
([ ([1;2;3], "a"); ([4;5], "b") ], "X") ;
([ ([6], "c"); ([7;8], "d"); ([9], "e")  ], "Y") ;
( [ ([10], "f") ]   , "Z")
]

  let l = flat_twice_flagged_list toto mk
  ------------------------------------------------------*)
let flat_twice_flagged_list 
    (inlist:   (('a list * 'b) list * 'c) list )
    (makeitem: 'a -> 'b -> 'c -> 'd ) 
    = (
      let g (dl: 'd list) ((albl: ('a list * 'b) list), (c: 'c)) = (
	let h (dl: 'd list) ((al: 'a list), (b: 'b)) = (
	  let f (a: 'a) = makeitem a b c in
	    List.append dl (List.map f al)
	) in
	  List.fold_left h dl albl
      ) in
	List.fold_left g [] inlist
    )

(**********************************************************************************)

(** Utilitaries to build [val_exp]  *)

let leafexp lxm op = CallByPos({src = lxm ; it = op }, Oper [])

let unexp lxm op e1 = CallByPos( {src = lxm ; it = op }, Oper [e1] )	
let unexp_predef lxm op e1 = CallByPos( {src = lxm ; it = Predef (op,[]) }, Oper [e1] )

let binexp lxm op e1 e2 = CallByPos( {src = lxm ; it = op }, Oper [e1 ; e2] ) 
let binexp_predef lxm op e1 e2 = CallByPos( {src = lxm ; it = Predef (op,[]) }, 
					    Oper [e1 ; e2] ) 

let ternexp lxm op e1 e2 e3 = CallByPos( {src = lxm ; it = op }, Oper [e1 ; e2; e3] )
let ternexp_predef lxm op e1 e2 e3 = CallByPos( {src = lxm ; it = Predef (op,[]) }, 
						Oper [e1 ; e2; e3] )

let naryexp lxm op elst = CallByPos( {src = lxm ; it = op }, Oper elst )	
let naryexp_predef lxm op elst = CallByPos( {src = lxm ; it = Predef (op,[]) }, 
					    Oper elst )


let bynameexp lxm op nelst = CallByName( {src = lxm ; it = op } , nelst )

open Ident


  
